import { use, setAttribute, effect, className, delegateEvents, template } from 'solid-js/web';
import { mergeProps, createSignal, onMount } from 'solid-js';

/* Original Code written by Oliver Geer (WebCoder49) */
// https://github.com/WebCoder49/code-input/blob/main/plugins/indent.js

function handleTabKey(event, input_element, code) {
  event.preventDefault(); // stop normal

  if (!event.shiftKey && input_element.selectionStart == input_element.selectionEnd) {
    // Shift always means dedent - this places a tab here.
    let before_selection = code.slice(0, input_element.selectionStart); // text before tab
    let after_selection = code.slice(input_element.selectionEnd, input_element.value.length); // text after tab

    let cursor_pos = input_element.selectionEnd + 1; // where cursor moves after tab - moving forward by 1 char to after tab
    input_element.value = before_selection + '\t' + after_selection; // add tab char

    // move cursor
    input_element.selectionStart = cursor_pos;
    input_element.selectionEnd = cursor_pos;
  } else {
    let lines = input_element.value.split('\n');
    let letter_i = 0;
    let selection_start = input_element.selectionStart; // where cursor moves after tab - moving forward by 1 indent
    let selection_end = input_element.selectionEnd; // where cursor moves after tab - moving forward by 1 indent

    let number_indents = 0;
    let first_line_indents = 0;
    for (let i = 0; i < lines.length; i++) {
      letter_i += lines[i].length + 1; // newline counted
      if (input_element.selectionStart <= letter_i && input_element.selectionEnd >= letter_i - lines[i].length) {
        // Starts before or at last char and ends after or at first char
        if (event.shiftKey) {
          if (lines[i][0] == '\t') {
            // Remove first tab
            lines[i] = lines[i].slice(1);
            if (number_indents == 0) first_line_indents--;
            number_indents--;
          }
        } else {
          lines[i] = '\t' + lines[i];
          if (number_indents == 0) first_line_indents++;
          number_indents++;
        }
      }
    }
    input_element.value = lines.join('\n');

    // move cursor
    input_element.selectionStart = selection_start + first_line_indents;
    input_element.selectionEnd = selection_end + number_indents;
  }
}
function handleEnterKey(event, input_element) {
  event.preventDefault();
  let lines = input_element.value.split('\n');
  let letter_i = 0;
  let current_line = lines.length - 1;
  let new_line = '';
  let number_indents = 0;

  // find the index of the line our cursor is currently on
  for (let i = 0; i < lines.length; i++) {
    letter_i += lines[i].length + 1; // newline counted
    if (input_element.selectionStart <= letter_i) {
      current_line = i;
      break;
    }
  }

  // count the number of indents the current line starts with (up to our cursor position in the line)
  let cursor_pos_in_line = lines[current_line].length - (letter_i - input_element.selectionEnd) + 1;
  for (let i = 0; i < cursor_pos_in_line; i++) {
    if (lines[current_line][i] == '\t') {
      number_indents++;
    } else {
      break;
    }
  }

  // determine the text before and after the cursor and chop the current line at the new line break
  let text_after_cursor = '';
  if (cursor_pos_in_line != lines[current_line].length) {
    text_after_cursor = lines[current_line].substring(cursor_pos_in_line);
    lines[current_line] = lines[current_line].substring(0, cursor_pos_in_line);
  }

  // insert our indents and any text from the previous line that might have been after the line break
  for (let i = 0; i < number_indents; i++) {
    new_line += '\t';
  }
  new_line += text_after_cursor;

  // save the current cursor position
  let selection_start = input_element.selectionStart;
  let selection_end = input_element.selectionEnd;

  // splice our new line into the list of existing lines and join them all back up
  lines.splice(current_line + 1, 0, new_line);
  input_element.value = lines.join('\n');

  // move cursor to new position
  input_element.selectionStart = selection_start + number_indents + 1; // count the indent level and the newline character
  input_element.selectionEnd = selection_end + number_indents + 1;
}

function styleInject(css, ref) {
  if (ref === void 0) ref = {};
  var insertAt = ref.insertAt;
  if (!css || typeof document === 'undefined') {
    return;
  }
  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';
  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }
  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = "/* Original CSS written by Oliver Geer (WebCoder49) */\n/* https://css-tricks.com/creating-an-editable-textarea-that-supports-syntax-highlighted-code/ */\n/* Updated for solid-code-input */\n\n.styles-module_code-input__ABQIg * {\n  box-sizing: border-box;\n}\n\n.styles-module_code-input__ABQIg {\n  box-sizing: border-box;\n  position: relative;\n  top: 0;\n  left: 0;\n  display: block;\n  overflow: hidden;\n  min-height: 76px;\n  font-size: 16px;\n  font-family: monospace;\n  line-height: normal;\n  tab-size: 2;\n  caret-color: darkgrey;\n  white-space: pre;\n}\n\n.styles-module_code-input__ABQIg.styles-module_resize-both__x-us- {\n  resize: both;\n}\n\n.styles-module_code-input__ABQIg.styles-module_resize-horizontal__Gt6Mt {\n  resize: horizontal;\n}\n\n.styles-module_code-input__ABQIg.styles-module_resize-vertical__eZhdp {\n  resize: vertical;\n}\n\n.styles-module_code-input__ABQIg textarea,\n.styles-module_code-input__ABQIg pre {\n  margin: 0 !important;\n  padding: 1rem !important;\n  border: 0;\n  width: 100%;\n  height: 100%;\n  position: absolute;\n  top: 0;\n  left: 0;\n}\n\n.styles-module_code-input__ABQIg textarea,\n.styles-module_code-input__ABQIg pre,\n.styles-module_code-input__ABQIg pre * {\n  font-size: inherit !important;\n  font-family: inherit !important;\n  line-height: inherit !important;\n  tab-size: inherit !important;\n}\n\n.styles-module_code-input__ABQIg textarea {\n  z-index: 1;\n  resize: none;\n  outline: none !important;\n}\n.styles-module_code-input__ABQIg pre {\n  z-index: 0;\n}\n\n.styles-module_code-input__ABQIg textarea {\n  color: transparent;\n  background: transparent;\n  caret-color: inherit !important;\n}\n\n.styles-module_code-input__ABQIg textarea,\n.styles-module_code-input__ABQIg pre {\n  overflow: auto !important;\n  white-space: inherit;\n  word-spacing: normal;\n  word-break: normal;\n  word-wrap: normal;\n}\n";
var styles = {"code-input":"styles-module_code-input__ABQIg","resize-both":"styles-module_resize-both__x-us-","resize-horizontal":"styles-module_resize-horizontal__Gt6Mt","resize-vertical":"styles-module_resize-vertical__eZhdp"};
styleInject(css_248z);

const _tmpl$ = /*#__PURE__*/template(`<div><textarea></textarea><pre><div class="code-highlighted"></div></pre></div>`, 8);
const CodeInput = props => {
  const merged = mergeProps({
    autoHeight: true
  }, props);
  let preElement;
  let textAreaElement;
  let wrapperElement;

  // Used to detect when the user manually resizes the wrapper with handle
  let wrapperHeight;
  let wrapperWidth;
  const [manualResize, setManualResize] = createSignal(false);
  onMount(async () => {
    watchResize();
    if (props.autoHeight) {
      autoHeight();
    }
  });
  function setSizes() {
    const {
      height,
      width
    } = getWrapperSize();
    preElement.style.width = `${width}px`;
    preElement.style.height = `${height}px`;
  }
  function getWrapperSize() {
    const {
      height,
      width
    } = wrapperElement.getBoundingClientRect();
    return {
      height,
      width
    };
  }
  function autoHeight() {
    if (manualResize() === true) {
      return;
    }
    wrapperElement.style.height = `0px`;
    wrapperElement.style.height = textAreaElement.scrollHeight + 'px';
  }
  function watchResize() {
    new ResizeObserver(setSizes).observe(wrapperElement);
  }
  function syncScroll() {
    preElement.scrollTop = textAreaElement.scrollTop;
    preElement.scrollLeft = textAreaElement.scrollLeft;
  }
  const codeTokens = () => {
    try {
      if (merged.prismJS) {
        if (merged.prismJS.languages[merged.language()]) {
          if (merged.autoHeight) {
            autoHeight();
          }
          return merged.prismJS.highlight(merged.value(), merged.prismJS.languages[merged.language()], merged.language());
        } else {
          if (merged.autoHeight) {
            autoHeight();
          }
          return merged.prismJS.util.encode(merged.value()).toString();
        }
      } else if (merged.highlightjs) {
        const tokens = merged.highlightjs.highlight(merged.value(), {
          language: merged.language()
        }).value;
        if (merged.autoHeight) {
          autoHeight();
        }
        return tokens;
      }
    } catch (e) {
      console.error(e);
    }
  };
  function handleKeyDown(event) {
    let input_element = textAreaElement;
    let code = input_element.value;
    if (event.key === 'Tab') {
      handleTabKey(event, input_element, code);
      merged.onChange(input_element.value);
    }
    if (event.key === 'Enter') {
      handleEnterKey(event, input_element);
      merged.onChange(input_element.value);
    }
  }
  async function handleInput(e) {
    merged.onChange(e.target.value);
  }
  function handleMouseDown() {
    const {
      height,
      width
    } = getWrapperSize();
    wrapperHeight = height;
    wrapperWidth = width;
  }
  function handleMouseUp() {
    const {
      height,
      width
    } = getWrapperSize();
    if (height !== wrapperHeight || width !== wrapperWidth) {
      setManualResize(true);
    }
  }
  return (() => {
    const _el$ = _tmpl$.cloneNode(true),
      _el$2 = _el$.firstChild,
      _el$3 = _el$2.nextSibling,
      _el$4 = _el$3.firstChild;
    const _ref$ = wrapperElement;
    typeof _ref$ === "function" ? use(_ref$, _el$) : wrapperElement = _el$;
    _el$.$$mouseup = handleMouseUp;
    _el$.$$mousedown = handleMouseDown;
    _el$2.$$input = handleInput;
    const _ref$2 = textAreaElement;
    typeof _ref$2 === "function" ? use(_ref$2, _el$2) : textAreaElement = _el$2;
    _el$2.addEventListener("scroll", syncScroll);
    _el$2.$$keydown = handleKeyDown;
    setAttribute(_el$2, "spellcheck", false);
    const _ref$3 = preElement;
    typeof _ref$3 === "function" ? use(_ref$3, _el$3) : preElement = _el$3;
    setAttribute(_el$3, "aria-hidden", true);
    effect(_p$ => {
      const _v$ = `${styles['code-input']} ${merged.resize ? styles[`resize-${merged.resize}`] : ''}`,
        _v$2 = merged.placeholder,
        _v$3 = `language-${merged.language()}`,
        _v$4 = codeTokens();
      _v$ !== _p$._v$ && className(_el$, _p$._v$ = _v$);
      _v$2 !== _p$._v$2 && setAttribute(_el$2, "placeholder", _p$._v$2 = _v$2);
      _v$3 !== _p$._v$3 && className(_el$3, _p$._v$3 = _v$3);
      _v$4 !== _p$._v$4 && (_el$4.innerHTML = _p$._v$4 = _v$4);
      return _p$;
    }, {
      _v$: undefined,
      _v$2: undefined,
      _v$3: undefined,
      _v$4: undefined
    });
    effect(() => _el$2.value = merged.value());
    return _el$;
  })();
};
delegateEvents(["mousedown", "mouseup", "keydown", "input"]);

export { CodeInput };
//# sourceMappingURL=index.js.map
